@@ -196,11 +196,6 @@
 
 ;;;; threading
 
-#!+(and sb-lutex sb-thread)
-(progn
-  (defknown sb!vm::%make-lutex () sb!vm::lutex ())
-  (defknown sb!vm::lutexp (t) boolean (foldable flushable)))
-
 (defknown (dynamic-space-free-pointer binding-stack-pointer-sap
                                       control-stack-pointer-sap)  ()
   system-area-pointer
