@@ -22,6 +22,8 @@
 
 ;;;; Check that we get debug arglists right.
 
+(defvar *p* (namestring *load-truename*))
+
 ;;; FIXME: This should use some get-argslist like functionality that
 ;;; we actually export.
 ;;;
@@ -106,7 +108,6 @@
                                   (caar frame-specs)
                                   full-backtrace)
                           (setf result nil))
-
                         ;; check that we have all the frames we wanted
                         (mapcar
                          (lambda (spec frame)
@@ -156,7 +157,7 @@
     (assert (verify-backtrace
              (lambda () (test #'optimized))
              (list *undefined-function-frame*
-                   (list '(flet test) #'optimized)))))
+                   (list `(flet test :in ,*p*) #'optimized)))))
 
   ;; bug 353: This test fails at least most of the time for x86/linux
   ;; ca. 0.8.20.16. -- WHN
@@ -170,11 +171,17 @@
     (assert (verify-backtrace
              (lambda () (test #'not-optimized))
              (list *undefined-function-frame*
-                   (list '(flet not-optimized))
-                   (list '(flet test) #'not-optimized))))))
+                   (list `(flet not-optimized :in ,*p*))
+                   (list `(flet test :in ,*p*) #'not-optimized))))))
 
 (with-test (:name :backtrace-interrupted-condition-wait
-            :skipped-on '(not :sb-thread))
+            :skipped-on '(not :sb-thread)
+                  ;; For some unfathomable reason the backtrace becomes
+                  ;; stunted on Darwin, ending at _sigtramp, when we add
+                  ;; :TIMEOUT NIL to the frame we expect. If we leave it out,
+                  ;; the backtrace is fine -- but the test fails. I can only
+                  ;; boggle right now.
+            :fails-on '(or (and :x86 :linux) :darwin))
   (let ((m (sb-thread:make-mutex))
         (q (sb-thread:make-waitqueue)))
     (assert (verify-backtrace
@@ -184,7 +191,7 @@
                                           (error "foo"))))
                   (with-timeout 0.1
                     (sb-thread:condition-wait q m)))))
-            `((sb-thread:condition-wait ,q ,m))))))
+            `((sb-thread:condition-wait ,q ,m :timeout nil))))))
 
 ;;; Division by zero was a common error on PPC. It depended on the
 ;;; return function either being before INTEGER-/-INTEGER in memory,
@@ -211,13 +218,13 @@
               :fails-on :alpha)  ; bug 346
     (assert (verify-backtrace (lambda () (test #'optimized))
                               (list '(/ 42 &rest)
-                                    (list '(flet test) #'optimized)))))
+                                    (list `(flet test :in ,*p*) #'optimized)))))
   (with-test (:name (:divide-by-zero :bug-356)
               :fails-on :alpha)  ; bug 356
     (assert (verify-backtrace (lambda () (test #'not-optimized))
                               (list '(/ 42 &rest)
-                                    '((flet not-optimized))
-                                    (list '(flet test) #'not-optimized))))))
+                                    `((flet not-optimized :in ,*p*))
+                                    (list `(flet test :in ,*p*) #'not-optimized))))))
 
 (with-test (:name (:throw :no-such-tag)
             :fails-on '(or
@@ -373,8 +380,8 @@
 (defvar *compile-nil-non-tc* (compile nil '(lambda (y) (cons (funcall *compile-nil-error* y) nil))))
 (with-test (:name (:compile nil))
   (assert (verify-backtrace (lambda () (funcall *compile-nil-non-tc* 13))
-                            '(((lambda (x)) 13)
-                              ((lambda (y)) 13)))))
+                            `(((lambda (x) :in ,*p*) 13)
+                              ((lambda (y) :in ,*p*) 13)))))
 
 (with-test (:name :clos-slot-typecheckfun-named)
   (assert
@@ -407,21 +414,21 @@
     (assert
      (verify-backtrace (lambda ()
                          (funcall (make-fun 0) 10 11 0))
-                       '((sb-kernel:two-arg-/ 10/11 0)
+                       `((sb-kernel:two-arg-/ 10/11 0)
                          (/ 10 11 0)
-                         ((lambda (&rest rest)) 10 11 0))))
+                         ((lambda (&rest rest) :in ,*p*) 10 11 0))))
     (assert
      (verify-backtrace (lambda ()
                          (funcall (make-fun 1) 10 11 0))
-                       '((sb-kernel:two-arg-/ 10/11 0)
+                       `((sb-kernel:two-arg-/ 10/11 0)
                          (/ 10 11 0)
-                         ((lambda (a &rest rest)) 10 11 0))))
+                         ((lambda (a &rest rest) :in ,*p*) 10 11 0))))
     (assert
      (verify-backtrace (lambda ()
                          (funcall (make-fun 2) 10 11 0))
-                       '((sb-kernel:two-arg-/ 10/11 0)
+                       `((sb-kernel:two-arg-/ 10/11 0)
                          (/ 10 11 0)
-                         ((lambda (a b &rest rest)) 10 11 0))))))
+                         ((lambda (a b &rest rest) :in ,*p*) 10 11 0))))))
 
 ;;;; test TRACE
 
@@ -494,7 +501,8 @@
       (declare (notinline dx-arg-backtrace))
       (assert (member-if (lambda (frame)
                            (and (consp frame)
-                                (equal '(flet dx-arg-backtrace) (car frame))
+                                (consp (car frame))
+                                (equal '(flet dx-arg-backtrace :in) (butlast (car frame)))
                                 (notany #'sb-debug::stack-allocated-p (cdr frame))))
                          (dx-arg-backtrace dx-arg))))))
 
@@ -558,5 +566,39 @@
 
 ;; unconditional, in case either previous left it enabled
 (disable-debugger)
+
+;;;; test some limitations of MAKE-LISP-OBJ
+
+;;; Older GENCGC systems had a bug in the pointer validation used by
+;;; MAKE-LISP-OBJ that made SIMPLE-FUN objects always fail to
+;;; validate.
+(with-test (:name (make-lisp-obj :simple-funs))
+  (sb-sys:without-gcing
+    (assert (eq #'identity
+                (sb-kernel:make-lisp-obj
+                 (sb-kernel:get-lisp-obj-address
+                  #'identity))))))
+
+;;; Older CHENEYGC systems didn't perform any real pointer validity
+;;; checks beyond "is this pointer to somewhere in heap space".
+(with-test (:name (make-lisp-obj :pointer-validation))
+  ;; Fun and games: We need to test MAKE-LISP-OBJ with a known-bogus
+  ;; address, but we also need the GC to not pitch a fit if it sees an
+  ;; object with said bogus address.  Thus, construct our known-bogus
+  ;; object within an area of unboxed storage (a vector) in static
+  ;; space.  We'll make it a simple object, (CONS 0 0), which has an
+  ;; in-memory representation of two consecutive zero words.  We
+  ;; allocate a three-word vector so that we can guarantee a
+  ;; double-word aligned double-word of zeros no matter what happens
+  ;; with the vector-data-offset (currently double-word aligned).
+  (let* ((memory (sb-int:make-static-vector 3 :element-type `(unsigned-byte ,sb-vm:n-word-bits)
+                                            :initial-element 0))
+         (vector-data-address (sb-sys:sap-int (sb-kernel::vector-sap memory)))
+         (object-base-address (logandc2 (+ vector-data-address sb-vm:lowtag-mask) sb-vm:lowtag-mask))
+         (object-tagged-address (+ object-base-address sb-vm:list-pointer-lowtag)))
+    (multiple-value-bind
+          (object valid-p)
+        (sb-kernel:make-lisp-obj object-tagged-address nil)
+      (assert (not valid-p)))))
 
 (write-line "/debug.impure.lisp done")
