@@ -37,10 +37,6 @@
     (with-mutex (mutex)
       mutex)))
 
-(with-test (:name (:with-spinlock :basics))
-  (let ((spinlock (make-spinlock)))
-    (with-spinlock (spinlock))))
-
 (sb-alien:define-alien-routine "check_deferrables_blocked_or_lose"
     void
   (where sb-alien:unsigned-long))
@@ -76,19 +72,19 @@
 
 ;;;; Now the real tests...
 
-(with-test (:name (:interrupt-thread :deferrables-unblocked-by-spinlock))
-  (let ((spinlock (sb-thread::make-spinlock))
+(with-test (:name (:interrupt-thread :deferrables-unblocked-by-lock))
+  (let ((lock (sb-thread::make-mutex))
         (thread (sb-thread:make-thread (lambda ()
                                          (loop (sleep 1))))))
-    (sb-thread::get-spinlock spinlock)
+    (sb-thread::grab-mutex lock)
     (sb-thread:interrupt-thread thread
                                 (lambda ()
                                   (check-deferrables-blocked-or-lose 0)
-                                  (sb-thread::get-spinlock spinlock)
+                                  (sb-thread::grab-mutex lock)
                                   (check-deferrables-unblocked-or-lose 0)
                                   (sb-ext:quit)))
     (sleep 1)
-    (sb-thread::release-spinlock spinlock)))
+    (sb-thread::release-mutex lock)))
 
 ;;; compare-and-swap
 
@@ -229,38 +225,11 @@
         (assert (ours-p (mutex-value l)) nil "5"))
       (assert (eql (mutex-value l) nil) nil "6"))))
 
-(with-test (:name (:with-recursive-spinlock :basics))
-  (labels ((ours-p (value)
-             (eq *current-thread* value)))
-    (let ((l (make-spinlock :name "rec")))
-      (assert (eql (spinlock-value l) nil) nil "1")
-      (with-recursive-spinlock (l)
-        (assert (ours-p (spinlock-value l)) nil "3")
-        (with-recursive-spinlock (l)
-          (assert (ours-p (spinlock-value l)) nil "4"))
-        (assert (ours-p (spinlock-value l)) nil "5"))
-      (assert (eql (spinlock-value l) nil) nil "6"))))
-
 (with-test (:name (:mutex :nesting-mutex-and-recursive-lock))
   (let ((l (make-mutex :name "a mutex")))
     (with-mutex (l)
       (with-recursive-lock (l)))))
 
-(with-test (:name (:spinlock :nesting-spinlock-and-recursive-spinlock))
-  (let ((l (make-spinlock :name "a spinlock")))
-    (with-spinlock (l)
-      (with-recursive-spinlock (l)))))
-
-(with-test (:name (:spinlock :more-basics))
-  (let ((l (make-spinlock :name "spinlock")))
-    (assert (eql (spinlock-value l) nil) ((spinlock-value l))
-            "spinlock not free (1)")
-    (with-spinlock (l)
-      (assert (eql (spinlock-value l) *current-thread*) ((spinlock-value l))
-              "spinlock not taken"))
-    (assert (eql (spinlock-value l) nil) ((spinlock-value l))
-            "spinlock not free (2)")))
-
 ;; test that SLEEP actually sleeps for at least the given time, even
 ;; if interrupted by another thread exiting/a gc/anything
 (with-test (:name (:sleep :continue-sleeping-after-interrupt))
@@ -342,8 +311,6 @@
                                       (grab-mutex m :waitp nil)))))))))
 
 (with-test (:name (:grab-mutex :timeout :acquisition-fail))
-  #+sb-lutex
-  (error "Mutex timeout not supported here.")
   (let ((m (make-mutex))
         (w (make-semaphore)))
     (with-mutex (m)
@@ -358,8 +325,6 @@
         (assert (null (join-thread th)))))))
 
 (with-test (:name (:grab-mutex :timeout :acquisition-success))
-  #+sb-lutex
-  (error "Mutex timeout not supported here.")
   (let ((m (make-mutex))
         (child))
     (with-mutex (m)
@@ -368,8 +333,6 @@
     (assert (eq (join-thread child) 't))))
 
 (with-test (:name (:grab-mutex :timeout+deadline))
-  #+sb-lutex
-  (error "Mutex timeout not supported here.")
   (let ((m (make-mutex))
         (w (make-semaphore)))
     (with-mutex (m)
@@ -384,8 +347,6 @@
         (assert (eq (join-thread th) :deadline))))))
 
 (with-test (:name (:grab-mutex :waitp+deadline))
-  #+sb-lutex
-  (error "Mutex timeout not supported here.")
   (let ((m (make-mutex)))
     (with-mutex (m)
       (assert (eq (join-thread
@@ -567,7 +528,8 @@
 
 (defun alloc-stuff () (copy-list '(1 2 3 4 5)))
 
-(with-test (:name (:interrupt-thread :interrupt-consing-child))
+(with-test (:name (:interrupt-thread :interrupt-consing-child)
+            :broken-on :darwin)
   (let ((thread (sb-thread:make-thread (lambda () (loop (alloc-stuff))))))
     (let ((killers
            (loop repeat 4 collect
@@ -1080,37 +1042,34 @@
 ;;; Make sure that a deadline handler is not invoked twice in a row in
 ;;; CONDITION-WAIT. See LP #512914 for a detailed explanation.
 ;;;
-#-sb-lutex    ; See KLUDGE above: no deadlines for condition-wait+lutexes.
 (with-test (:name (:condition-wait :deadlines :LP-512914))
-  (let ((n 2) ; was empirically enough to trigger the bug
+  (let ((n 2)                      ; was empirically enough to trigger the bug
         (mutex (sb-thread:make-mutex))
         (waitq (sb-thread:make-waitqueue))
         (threads nil)
         (deadline-handler-run-twice? nil))
     (dotimes (i n)
       (let ((child
-             (sb-thread:make-thread
-              #'(lambda ()
-                  (handler-bind
-                      ((sb-sys:deadline-timeout
-                        (let ((already? nil))
-                          #'(lambda (c)
-                              (when already?
-                                (setq deadline-handler-run-twice? t))
-                              (setq already? t)
-                              (sleep 0.2)
-                              (sb-thread:condition-broadcast waitq)
-                              (sb-sys:defer-deadline 10.0 c)))))
-                    (sb-sys:with-deadline (:seconds 0.1)
-                      (sb-thread:with-mutex (mutex)
-                        (sb-thread:condition-wait waitq mutex))))))))
+              (sb-thread:make-thread
+               #'(lambda ()
+                   (handler-bind
+                       ((sb-sys:deadline-timeout
+                          (let ((already? nil))
+                            #'(lambda (c)
+                                (when already?
+                                  (setq deadline-handler-run-twice? t))
+                                (setq already? t)
+                                (sleep 0.2)
+                                (sb-thread:condition-broadcast waitq)
+                                (sb-sys:defer-deadline 10.0 c)))))
+                     (sb-sys:with-deadline (:seconds 0.1)
+                       (sb-thread:with-mutex (mutex)
+                         (sb-thread:condition-wait waitq mutex))))))))
         (push child threads)))
     (mapc #'sb-thread:join-thread threads)
     (assert (not deadline-handler-run-twice?))))
 
 (with-test (:name (:condition-wait :signal-deadline-with-interrupts-enabled))
-  #+darwin
-  (error "Bad Darwin")
   (let ((mutex (sb-thread:make-mutex))
         (waitq (sb-thread:make-waitqueue))
         (A-holds? :unknown)
@@ -1136,7 +1095,8 @@
                            (sb-sys:defer-deadline 10.0 c))))
                    (sb-sys:with-deadline (:seconds 0.1)
                      (sb-thread:with-mutex (mutex)
-                       (sb-thread:condition-wait waitq mutex)))))))
+                       (sb-thread:condition-wait waitq mutex)))))
+             :name "A"))
     (setq B (sb-thread:make-thread
              #'(lambda ()
                  (thread-yield)
@@ -1155,7 +1115,8 @@
                            (sb-sys:defer-deadline 10.0 c))))
                    (sb-sys:with-deadline (:seconds 0.1)
                      (sb-thread:with-mutex (mutex)
-                       (sb-thread:condition-wait waitq mutex)))))))
+                       (sb-thread:condition-wait waitq mutex)))))
+             :name "B"))
     (sb-thread:join-thread A)
     (sb-thread:join-thread B)
     (let ((A-result (list A-holds? A-interrupts-enabled?))
@@ -1167,7 +1128,10 @@
       ;; behaviour.
       (cond ((equal A-result '(t t)) (assert (equal B-result '(nil t))))
             ((equal B-result '(t t)) (assert (equal A-result '(nil t))))
-            (t (error "Failure: fall through."))))))
+            (t
+             (error "Failure: fell through wit A: ~S, B: ~S"
+                    A-result
+                    B-result))))))
 
 (with-test (:name (:mutex :finalization))
   (let ((a nil))
@@ -1209,9 +1173,7 @@
 
 (format t "infodb test done~%")
 
-(with-test (:name (:backtrace))
-  #+darwin
-  (error "Prone to crash on Darwin, cause unknown.")
+(with-test (:name :backtrace :broken-on :darwin)
   ;; Printing backtraces from several threads at once used to hang the
   ;; whole SBCL process (discovered by accident due to a timer.impure
   ;; test misbehaving). The cause was that packages weren't even
@@ -1230,9 +1192,9 @@
 
 (format t "~&starting gc deadlock test: WARNING: THIS TEST WILL HANG ON FAILURE!~%")
 
-(with-test (:name (:gc-deadlock))
-  #+darwin
-  (error "Prone to hang on Darwin due to interrupt issues.")
+(with-test (:name :gc-deadlock
+                  ;; Prone to hang on Darwin due to interrupt issues.
+            :broken-on :darwin)
   ;; Prior to 0.9.16.46 thread exit potentially deadlocked the
   ;; GC due to *all-threads-lock* and session lock. On earlier
   ;; versions and at least on one specific box this test is good enough
